/*
 * Copyright (C) 2011 Atmark Techno, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/i2c.h>
#include <linux/err.h>
#include <linux/pmic_external.h>
#include <linux/regulator/machine.h>
#include <linux/regulator/fixed.h>
#include <linux/mfd/mc34704/core.h>
#include <mach/common.h>
#include "board-armadillo400.h"

/*
 * Convenience conversion.
 * Here atm, maybe there is somewhere better for this.
 */
#define mV_to_uV(mV) (mV * 1000)
#define uV_to_mV(uV) (uV / 1000)
#define V_to_uV(V) (mV_to_uV(V * 1000))
#define uV_to_V(uV) (uV_to_mV(uV) / 1000)

/*
 * Fixed Regulator Settings
 */
#if defined(CONFIG_ARMADILLO400_SELECTED_SDHC1)
static struct regulator_consumer_supply armadillo400_sdhc1_power_consumers[] = {
	REGULATOR_SUPPLY("sdhc1-power", "mxsdhci.0"),
};

static struct regulator_init_data armadillo400_sdhc1_power_init_data = {
	.supply_regulator = "REG2_CPU",
	.constraints = {
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
	},
	.consumer_supplies = armadillo400_sdhc1_power_consumers,
	.num_consumer_supplies = ARRAY_SIZE(armadillo400_sdhc1_power_consumers),
};

static struct fixed_voltage_config armadillo400_sdhc1_power_config = {
	.supply_name = "sdhc1.pwrsw",
	.gpio = SDHC1_PWREN_GPIO,
	.enable_high = 0,
	.enabled_at_boot = 0,
	.startup_delay = 100000,
	.microvolts = 3300000,
	.init_data = &armadillo400_sdhc1_power_init_data,
};

static struct platform_device armadillo400_sdhc1_power = {
	.name = "reg-fixed-voltage",
	.id = 0,
};
#endif

#if defined(CONFIG_ARMADILLO400_SDHC2_PWREN_CON9_1)
static struct regulator_consumer_supply armadillo400_sdhc2_power_consumers[] = {
	REGULATOR_SUPPLY("sdhc2-power", "mxsdhci.1"),
};

static struct regulator_init_data armadillo400_sdhc2_power_init_data = {
	.supply_regulator = "REG5_PERS",
	.constraints = {
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
	},
	.consumer_supplies = armadillo400_sdhc2_power_consumers,
	.num_consumer_supplies = ARRAY_SIZE(armadillo400_sdhc2_power_consumers),
};

static struct fixed_voltage_config armadillo400_sdhc2_power_config = {
	.supply_name = "sdhc2.pwrsw",
	.gpio = SDHC2_PWREN_GPIO,
	.enable_high = 0,
	.enabled_at_boot = 0,
	.startup_delay = 100000,
	.microvolts = 3300000,
	.init_data = &armadillo400_sdhc2_power_init_data,
};

static struct platform_device armadillo400_sdhc2_power = {
	.name = "reg-fixed-voltage",
	.id = 1,
};
#endif

static struct regulator_consumer_supply armadillo400_usb_vin_power_consumers[] = {
	REGULATOR_SUPPLY("usb-vin-power", "fsl-ehci.0"),
	REGULATOR_SUPPLY("usb-vin-power", "fsl-ehci.1"),
};

static struct regulator_init_data armadillo400_usb_vin_power_init_data = {
	.constraints = {
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
	},
	.consumer_supplies = armadillo400_usb_vin_power_consumers,
	.num_consumer_supplies = ARRAY_SIZE(armadillo400_usb_vin_power_consumers),
};

static struct fixed_voltage_config armadillo400_usb_vin_power_config = {
	.supply_name = "USB_VIN",
	.gpio = USB_PWRSEL_GPIO,
	.enable_high = 0,
	.enabled_at_boot = 0,
	.startup_delay = 100000,
	.microvolts = 5000000,
	.init_data = &armadillo400_usb_vin_power_init_data,
};

static struct platform_device armadillo400_usb_vin_power = {
	.name = "reg-fixed-voltage",
	.id = 2,
};

/*
 * MC34704 Regulator Settings
 */
static struct regulator_consumer_supply reg1_bklt_consumers[] = {
	REGULATOR_SUPPLY("usb-pmic-power", "fsl-ehci.0"),
	REGULATOR_SUPPLY("usb-pmic-power", "fsl-ehci.1"),
};

static struct regulator_init_data rbklt_init = {
	.constraints = {
		.name = "REG1_BKLT",
		.min_uV = mV_to_uV(REG1_V_MV),
		.max_uV = mV_to_uV(REG1_V_MV),
		.valid_ops_mask = (REGULATOR_CHANGE_VOLTAGE |
				   REGULATOR_CHANGE_STATUS),
	},
	.consumer_supplies = reg1_bklt_consumers,
	.num_consumer_supplies = ARRAY_SIZE(reg1_bklt_consumers),
};

static struct regulator_consumer_supply reg2_rcpu_consumers[] = {
	REGULATOR_SUPPLY("uart2-power", "mxcintuart.1"),
};

static struct regulator_init_data rcpu_init = {
	.constraints = {
		.name = "REG2_CPU",
		.min_uV = mV_to_uV(REG2_V_MV),
		.max_uV = mV_to_uV(REG2_V_MV),
		.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
	},
	.consumer_supplies = reg2_rcpu_consumers,
	.num_consumer_supplies = ARRAY_SIZE(reg2_rcpu_consumers),
};

static struct regulator_init_data rcore_init = {
	.constraints = {
		.name = "REG3_CORE",
		.min_uV = mV_to_uV(REG3_V_MV),
		.max_uV = mV_to_uV(REG3_V_MV),
		.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
	},
};

static struct regulator_init_data rddr_init = {
	.constraints = {
		.name = "REG4_DDR",
		.min_uV = mV_to_uV(REG4_V_MV),
		.max_uV = mV_to_uV(REG4_V_MV),
		.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
	},
};

static struct regulator_consumer_supply reg5_pers_consumers[] = {
#if defined(CONFIG_ARMADILLO400_UART3_CON9)
	REGULATOR_SUPPLY("uart3-power", "mxcintuart.2"),
#endif
#if defined(CONFIG_ARMADILLO400_UART4_CON11)
	REGULATOR_SUPPLY("uart4-power", "mxcintuart.3"),
#endif
#if defined(CONFIG_ARMADILLO400_UART5_CON9)
	REGULATOR_SUPPLY("uart5-power", "mxcintuart.4"),
#endif
#if defined(CONFIG_ARMADILLO400_I2C2_CON14_S35390A)
	REGULATOR_SUPPLY("s35390a-power", "1-0030"),
#endif
#if defined(CONFIG_ARMADILLO400_I2C3_CON11_S35390A)
	REGULATOR_SUPPLY("s35390a-power", "2-0030"),
#endif
	REGULATOR_SUPPLY("lcd-power", "mxc_sdc_fb.0"),
	REGULATOR_SUPPLY("ts-power", "imx_adc_ts.0"),
#if defined(CONFIG_ARMADILLO400_AUD5_CON11_WM8978)
	REGULATOR_SUPPLY("wm8978-power", "2-001a"),
#endif
#if defined(CONFIG_ARMADILLO400_AUD6_CON9_WM8978)
	REGULATOR_SUPPLY("wm8978-power", "1-001a"),
#endif
};

static struct regulator_init_data rpers_init = {
	.constraints = {
		.name = "REG5_PERS",
		.min_uV = mV_to_uV(REG5_V_MV),
		.max_uV = mV_to_uV(REG5_V_MV),
		.valid_ops_mask = (REGULATOR_CHANGE_VOLTAGE |
				   REGULATOR_CHANGE_STATUS),
	},
	.consumer_supplies = reg5_pers_consumers,
	.num_consumer_supplies = ARRAY_SIZE(reg5_pers_consumers),
};

static int __devinitdata reg1_temporarily_enabled = 0;
static int __devinitdata reg5_temporarily_enabled = 0;

static int __devinit armadillo400_regulator_init(struct mc34704 *mc34704)
{
	struct regulator *regulator;

	mc34704_register_regulator(mc34704, MC34704_BKLT, &rbklt_init);
	mc34704_register_regulator(mc34704, MC34704_CPU, &rcpu_init);
	mc34704_register_regulator(mc34704, MC34704_CORE, &rcore_init);
	mc34704_register_regulator(mc34704, MC34704_DDR, &rddr_init);
	mc34704_register_regulator(mc34704, MC34704_PERS, &rpers_init);

	regulator = regulator_get(NULL, "REG1_BKLT");
	if (regulator) {
		if (regulator_is_enabled(regulator)) {
			regulator_enable(regulator);
			reg1_temporarily_enabled = 1;
		}
	}
	regulator = regulator_get(NULL, "REG5_PERS");
	if (regulator) {
		if (regulator_is_enabled(regulator)) {
			regulator_enable(regulator);
			reg5_temporarily_enabled = 1;
		}
	}

#if defined(CONFIG_ARMADILLO400_SELECTED_SDHC1)
	mxc_register_device(&armadillo400_sdhc1_power,
			    &armadillo400_sdhc1_power_config);
#endif
#if defined(CONFIG_ARMADILLO400_SDHC2_PWREN_CON9_1)
	mxc_register_device(&armadillo400_sdhc2_power,
			    &armadillo400_sdhc2_power_config);
#endif
	mxc_register_device(&armadillo400_usb_vin_power,
			    &armadillo400_usb_vin_power_config);

	return 0;
}

static int __devinit mc34704_regulator_late_init(void)
{
	struct regulator *regulator;

	if (reg1_temporarily_enabled) {
		regulator = regulator_get(NULL, "REG1_BKLT");
		regulator_disable(regulator);
	}
	if (reg5_temporarily_enabled) {
		regulator = regulator_get(NULL, "REG5_PERS");
		regulator_disable(regulator);
	}
	return 0;
}
late_initcall(mc34704_regulator_late_init);

struct mc34704_platform_data __devinitdata mc34704_plat = {
	.init = armadillo400_regulator_init,
};
