/*
 * Copyright (C) 2011 Atmark Techno, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/kthread.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include "watchdog.h"

#define DRIVER_NAME "mx25-wdt"

#define WDOG_WCR	0	/* 16bit watchdog control reg */
#define WDOG_WSR	2	/* 16bit watchdog service reg */
#define WDOG_WRSR	4	/* 16bit watchdog reset status reg */

static const void *wdog_base = MX25_IO_ADDRESS(MX25_WDOG_BASE_ADDR);

struct mx25_wdt {
	unsigned int interval_ms;
	struct task_struct *task;
};

static int mx25_wdt_kick(void *data)
{
	struct mx25_wdt *wdt = data;

	do {
		/* issue the service sequence instructions */
		__raw_writew(0x5555, wdog_base + WDOG_WSR);
		__raw_writew(0xAAAA, wdog_base + WDOG_WSR);

		msleep(wdt->interval_ms);
	} while (!kthread_should_stop());

	return 0;
}

static int __init mx25_wdt_probe(struct platform_device *pdev)
{
	struct mx25_wdt_platform_data *pdata = pdev->dev.platform_data;
	struct mx25_wdt *wdt;
	struct task_struct *k;

	wdt = kzalloc(sizeof(struct mx25_wdt), GFP_KERNEL);
	if (wdt == NULL)
		return -ENOMEM;

	k = kthread_run(mx25_wdt_kick, wdt, DRIVER_NAME);
	if (IS_ERR(k)) {
		kfree(wdt);
		return PTR_ERR(k);
	}

	wdt->task = k;
	wdt->interval_ms = pdata->interval_ms;
	platform_set_drvdata(pdev, wdt);

	return 0;
}

static int __exit mx25_wdt_remove(struct platform_device *pdev)
{
	struct mx25_wdt *wdt = platform_get_drvdata(pdev);

	kthread_stop(wdt->task);

	return 0;
}

static struct platform_driver mx25_wdt_driver = {
	.remove		= __exit_p(mx25_wdt_remove),
	.driver		= {
		.name	= DRIVER_NAME,
		.owner	= THIS_MODULE,
	},
};

static int __init mx25_wdt_init(void)
{
	return platform_driver_probe(&mx25_wdt_driver, mx25_wdt_probe);
}
module_init(mx25_wdt_init);

static void __exit mx25_wdt_exit(void)
{
	platform_driver_unregister(&mx25_wdt_driver);
}
module_exit(mx25_wdt_exit);

MODULE_AUTHOR("Atmark Techno, Inc.");
MODULE_DESCRIPTION("Watchdog driver for i.MX25");
MODULE_LICENSE("GPL v2");
